/*
    SPDX-FileCopyrightText: 2009 Aleix Pol <aleixpol@kde.org>

    SPDX-License-Identifier: LGPL-2.0-or-later
*/

#include "idocumentationcontroller.h"

namespace KDevelop
{

IDocumentationController::IDocumentationController()
{
}

IDocumentationController::~IDocumentationController()
{
}

}

#include "moc_idocumentationcontroller.cpp"
